" ########
" # Plug #
" ########

call plug#begin('~/.local/share/nvim/plugged')

Plug 'chriskempson/base16-vim'
Plug 'dracula/vim', { 'as': 'dracula' }
Plug 'altercation/vim-colors-solarized'
Plug 'arcticicestudio/nord-vim'

Plug 'itchyny/lightline.vim'
Plug 'daviesjamie/vim-base16-lightline'
Plug 'maximbaz/lightline-ale'

Plug 'gioele/vim-autoswap' "Requires wmctrl

Plug 'sheerun/vim-polyglot'
Plug 'sirtaj/vim-openscad'
Plug 'othree/yajs.vim'
Plug 'othree/javascript-libraries-syntax.vim'
Plug 'jparise/vim-graphql'
Plug 'styled-components/vim-styled-components'

Plug 'vim-pandoc/vim-pandoc-syntax'

Plug 'junegunn/fzf.vim'
Plug 'scrooloose/nerdtree'
Plug 'xuyuanp/nerdtree-git-plugin'

Plug 'airblade/vim-gitgutter'

Plug 'nathanaelkane/vim-indent-guides'

" Plug 'donRaphaco/neotex', { 'for': 'tex' }
Plug 'lervag/vimtex'

Plug 'mhinz/neovim-remote'
if has('nvim')
  Plug 'Shougo/deoplete.nvim', { 'do': ':UpdateRemotePlugins' }
else
  Plug 'Shougo/deoplete.nvim'
  Plug 'roxma/nvim-yarp'
  Plug 'roxma/vim-hug-neovim-rpc'
endif
Plug 'Shougo/neco-syntax'
Plug 'Shougo/neoinclude.vim'
Plug 'Shougo/neopairs.vim'
Plug 'Shougo/context_filetype.vim'
Plug 'Shougo/neco-vim'
Plug 'carlitux/deoplete-ternjs', { 'do': 'npm install -g tern' }
Plug 'ternjs/tern_for_vim'

Plug 'Shougo/neosnippet.vim'
Plug 'Shougo/neosnippet-snippets'

Plug 'w0rp/ale'

call plug#end()

" ###########
" # General #
" ###########

let g:python_host_prog = '/usr/bin/python2'
let g:python3_host_prog = '/usr/bin/python'

" Syntax
filetype plugin indent on
let g:polyglot_disabled = ['js', 'latex']
syntax on

" Colorscheme
let base16colorspace=256
let g:dracula_colorterm=0
let g:nord_underline = 1
let g:nord_italic = 1
let g:nord_italic_comments = 0
let g:nord_cursor_line_number_background = 1
let g:solarized_termcolors= 16 
let g:solarized_bold = 1 
let g:solarized_underline = 1 
let g:solarized_italic = 1

colorscheme {{VIMCOLORSCHEME}}
set bg={{VIMBACKGROUND}}

" make the highlighting of tabs and other non-text less annoying
highlight SpecialKey ctermbg=none ctermfg=8
highlight NonText ctermbg=none ctermfg=8

" Tabs and Identation
set expandtab
set copyindent
set preserveindent
set softtabstop=2
set shiftwidth=2
set tabstop=2

" set list
" set lcs+=space:·

" Essentials
set encoding=utf-8                              " Set encoding
set ffs=unix,dos,mac
set title titlestring=                          " Change the terminal's title
set number                                      " Show line numbers
set relativenumber                              " Show relative numbers
set cursorline                                  " Highlight current line
set showcmd                                     " Show command in bottom bar
set wildmenu                                    " Visual autocomplete for command menu
set wildignore+=*/tmp/*,*.so,*.swp,*.zip        " MacOSX/Linux
"set wildignore+=*\\tmp\\*,*.swp,*.zip,*.exe    " Windows
set timeoutlen=1000 ttimeoutlen=0               " No timeout before keypresses
set scrolloff=10

set lazyredraw          " Redraw only when we need to.
set showmatch           " Highlight matching [{()}]

" History
set history=1000        " Remember more commands and search history
set undolevels=1000     " Use many muchos levels of undo

" Search
set incsearch           " Search as characters are entered
set hlsearch            " Highlight matches
set ignorecase          " Ignore case when searching
set smartcase           " When searching try to be smart about cases

" Folding
set foldenable          " Enable folding
set foldlevelstart=99   " Don't fold things by default
set foldcolumn=0        " Don't show fold column
set foldmethod=indent
set foldopen=block,hor,insert,jump,mark,percent,quickfix,search,tag,undo

" Mouse support
set mouse=a

" Backups
set backup
set backupdir=~/.vim-tmp,~/.tmp,~/tmp,/var/tmp,/tmp
set backupskip=/tmp/*,/private/tmp/*
set directory=~/.vim-tmp,~/.tmp,~/tmp,/var/tmp,/tmp
set writebackup

" ###########
" # Keymaps #
" ###########

" Leader
let mapleader=","       

" Fold toggle
nnoremap <space> za     

" Movement
nnoremap j gj
nnoremap k gk

" Turn off search highlight
nnoremap <leader><space> :nohlsearch<CR>

" Session save
nnoremap <leader>s :mksession<CR>

" Make p in Visual mode replace the selected text with the yank register
vnoremap p <Esc>:let current_reg = @"<CR>gvdi<C-R>=current_reg<CR><Esc>

" :W sudo saves the file
command W w !sudo tee % > /dev/null

" Completion with tab
inoremap <expr> <Tab> pumvisible() ? "\<C-n>" : "\<Tab>"
inoremap <expr> <S-Tab> pumvisible() ? "\<C-p>" : "\<S-Tab>"

" NERD Tree
map <C-n> :NERDTreeToggle<CR>

" Ale
nmap <silent> <leader>aj :ALENext<cr>
nmap <silent> <leader>ak :ALEPrevious<cr>

" fzf
nnoremap <c-p> :Files<cr>

" #############
" # Statusbar #
" #############

set laststatus=2
set noshowmode

let g:lightline = {
      \ 'colorscheme': '{{VIMLIGHTLINE}}',
      \ 'component_expand': {
      \      'linter_checking': 'lightline#ale#checking',
      \      'linter_warnings': 'lightline#ale#warnings',
      \      'linter_errors': 'lightline#ale#errors',
      \      'linter_ok': 'lightline#ale#ok',
      \ },
      \ 'component_type': {
      \     'linter_checking': 'left',
      \     'linter_warnings': 'warning',
      \     'linter_errors': 'error',
      \     'linter_ok': 'left',
      \ },
      \ 'active': {
      \     'right': [ [ 'linter_checking', 'linter_errors', 'linter_warnings', 'linter_ok' ],
      \                [ 'percent',  'lineinfo'],
      \                [ 'fileformat', 'fileencoding', 'filetype'] ]
      \  },
      \ }

" ###########
" # Plugins #
" ###########

" Indent guides
hi IndentGuidesOdd  ctermbg=8
hi IndentGuidesEven ctermbg=8
let g:indent_guides_start_level = 2
let g:indent_guides_guide_size = 1
let g:indent_guides_enable_on_vim_startup = 1

" Deoplete
let g:deoplete#enable_at_startup = 1
let g:deoplete#num_processes = 1

"call deoplete#custom#option('sources', {
"		\ '_': ['buffer'],
"		\ 'cpp': ['buffer', 'tag'],
"		\})

let g:tern#command = ["tern"]
let g:tern#arguments = ["--persistent"]

" ale
let g:ale_completion_enabled = 0
let g:ale_linters_explicit = 1
let g:ale_linters = {
      \   'cpp': ['clangcheck', 'clangtidy', 'cppcheck'],
      \   'javascript': ['eslint'],
      \   'vim': ['vint'],
      \}

let b:ale_fixers = {
      \ 'javascript': ['eslint']
      \}

" neotex
let g:neotex_enabled	= 1
let g:tex_flavor = 'latex'
let g:neotex_pdflatex_alternative	= 'xelatex'

" neopairs
let g:neopairs#enable = 1

" neco-vim
if !exists('g:necovim#complete_functions')
		  let g:necovim#complete_functions = {}
		endif
		let g:necovim#complete_functions.Ref =
		\ 'ref#complete'


" neosnippet

" Plugin key-mappings.
" Note: It must be "imap" and "smap".  It uses <Plug> mappings.
imap <C-k>     <Plug>(neosnippet_expand_or_jump)
smap <C-k>     <Plug>(neosnippet_expand_or_jump)
xmap <C-k>     <Plug>(neosnippet_expand_target)

" SuperTab like snippets behavior.
" Note: It must be "imap" and "smap".  It uses <Plug> mappings.
"imap <expr><TAB>
" \ pumvisible() ? "\<C-n>" :
" \ neosnippet#expandable_or_jumpable() ?
" \    "\<Plug>(neosnippet_expand_or_jump)" : "\<TAB>"
smap <expr><TAB> neosnippet#expandable_or_jumpable() ?
\ "\<Plug>(neosnippet_expand_or_jump)" : "\<TAB>"


" For conceal markers.
"if has('conceal')
"  set conceallevel=2 concealcursor=niv
"endif

let g:neosnippet#snippets_directory='~/.config/nvim/snippets'
